#include <LSS.h>
#include <ppm.h>

LSS myLSS1 = LSS(1);
LSS myLSS2 = LSS(2);
LSS myLSS3 = LSS(3);
LSS myLSS4 = LSS(4);
LSS myLSS5 = LSS(5);

float initPos1 = 0;
float initPos2 = 0;
float initPos3 = 0;
float initPos4 = 0;
float initPos5 = 0;

float currPos1, prevPos1, targPos1;
float currPos2, prevPos2, targPos2;
float currPos3, prevPos3, targPos3;
float currPos4, prevPos4, targPos4;
float currPos5, prevPos5, targPos5;

#define ROLL            1
// #define PITCH           2
#define THROTTLE        3
// #define YAW             4
// #define SWITCH3WAY_1    5
// #define BUTTON          6
// #define SWITCH3WAY_2    7
// #define POT             8

byte i;   // increment() counter
byte ird; // increment rate divider
byte state;
byte prevState;

byte phase = 1;
byte phaseDelay = 250;
byte queryDelay = 2;

bool wakeUpDone = false; // to indicate if wakeUp() has reached its target positions

void setup()
{
  ppm.begin(A3, false);

  LSS::initBus(Serial, 115200);

  // Enable Motion Profile
  Serial.write("#254CEM0\r");  // EM0 = motion profile disabled

  // Filter Position Count
  Serial.write("#254FPC10");   // default = 5

  // Angular Stiffness
  Serial.write("#254CAS-2\r"); // default = 0

  // Angular Holding Stiffness
  Serial.write("#254CAH-2\r"); // default = 4

  // LED Color
  // Serial.write("#254CLED3\r"); // 3 = blue

  Serial.write("#254RESET\r");
  delay(2000); // wait 2 sec for reboot

  queryPos();  // get positions at startup
  setPrevPos();
}

void loop()
{
  if (wakeUpDone == false)
  {
    wakeUp();
  }

  else if (wakeUpDone == true)
  {
    stateMachine();
    switch (state)
    {
      case 0:
        Stop();   // move to initial positions
        break;

      case 1:
        move_F(); // move forward
        break;

      case 2:
        move_B(); // move backward
        break;
    }
  }
}

void queryPos() // query positions and set them as current positions
{
  currPos1 = myLSS1.getPosition(); delay(queryDelay);
  currPos2 = myLSS2.getPosition(); delay(queryDelay);
  currPos4 = myLSS4.getPosition(); delay(queryDelay);
  currPos5 = myLSS5.getPosition(); delay(queryDelay);
}

void wakeUp()
{
  ird = 100;

  targPos1 =  initPos1;
  targPos2 =  initPos2;
  targPos4 =  initPos4;
  targPos5 =  initPos5;

  if (i < ird)
  {
    increment();
    i++;
  }

  else
  {
    myLSS3.move(initPos3);
    setPrevPos();
    wakeUpDone = true;
    delay(1000);
    i = 0;
  }

  updatePos();
}

void stateMachine()
{
  short roll =  map(ppm.read_channel(ROLL), 1000, 2000, -450, 450);
  short throttle = ppm.read_channel(THROTTLE);

  myLSS3.move(roll);

  if (throttle >= 1600) // move_F()
  {
    if (prevState == 1)
    {
      state = 1;
    }

    else if (prevState != 1)
    {
      Stop();
      state = prevState = 1;
    }
  }

  else if (throttle <= 1400) // move_B()
  {
    if (prevState == 2)
    {
      state = 2;
    }

    else if (prevState != 2)
    {
      Stop();
      state = prevState = 2;
    }
  }

  else
  {
    state = prevState = 0;
  }

}

void move_F()
{
  if (phase == 1)
  {
    ird = 10;

    targPos1 = initPos1;
    targPos2 = initPos2;
    targPos4 = initPos4;
    targPos5 =     -450;

    if (i < ird)
    {
      increment();
      i++;
    }

    else
    {
      delay(phaseDelay);
      setPrevPos();
      phase = 2;
      i = 0;
    }
  }

  else if (phase == 2)
  {
    ird = 100;

    targPos1 =   675;
    targPos2 =  -675;
    targPos4 =   675;
    targPos5 = -1125;

    if (i < ird)
    {
      increment();
      i++;
    }

    else
    {
      delay(phaseDelay);
      setPrevPos();
      phase = 3;
      i = 0;
    }
  }

  else if (phase == 3)
  {
    ird = 10;

    targPos1 = 1125;
    targPos2 = -675;
    targPos4 =  675;
    targPos5 = -675;

    if (i < ird)
    {
      increment();
      i++;
    }

    else
    {
      delay(phaseDelay);
      setPrevPos();
      phase = 4;
      i = 0;
    }
  }

  else if (phase == 4)
  {
    ird = 100;

    targPos1 =      450;
    targPos2 = initPos2;
    targPos4 = initPos4;
    targPos5 = initPos5;

    if (i < ird)
    {
      increment();
      i++;
    }

    else
    {
      delay(phaseDelay);
      setPrevPos();
      phase = 1;
      i = 0;
    }
  }

  updatePos();
}

void move_B()
{
  if (phase == 1)
  {
    ird = 10;

    targPos1 =      450;
    targPos2 = initPos2;
    targPos4 = initPos4;
    targPos5 = initPos5;

    if (i < ird)
    {
      increment();
      i++;
    }

    else
    {
      delay(phaseDelay);
      setPrevPos();
      phase = 2;
      i = 0;
    }
  }

  else if (phase == 2)
  {
    ird = 100;

    targPos1 =  1125;
    targPos2 =  -675;
    targPos4 =   675;
    targPos5 =  -675;

    if (i < ird)
    {
      increment();
      i++;
    }

    else
    {
      delay(phaseDelay);
      setPrevPos();
      phase = 3;
      i = 0;
    }
  }

  else if (phase == 3)
  {
    ird = 10;

    targPos1 =   675;
    targPos2 =  -675;
    targPos4 =   675;
    targPos5 = -1125;

    if (i < ird)
    {
      increment();
      i++;
    }

    else
    {
      delay(phaseDelay);
      setPrevPos();
      phase = 4;
      i = 0;
    }
  }

  else if (phase == 4)
  {
    ird = 100;

    targPos1 = initPos1;
    targPos2 = initPos2;
    targPos4 = initPos4;
    targPos5 =     -450;

    if (i < ird)
    {
      increment();
      i++;
    }

    else
    {
      delay(phaseDelay);
      setPrevPos();
      phase = 1;
      i = 0;
    }
  }

  updatePos();
}

void Stop()
{
  ird = 40;

  targPos1 =  initPos1;
  targPos2 =  initPos2;
  targPos4 =  initPos4;
  targPos5 =  initPos5;

  if (i < ird)
  {
    increment();
    i++;
  }

  else
  {
    delay(phaseDelay);
    setPrevPos();
    phase = 1;
    i = 0;
  }

  updatePos();
}

void increment() // divides the difference between target positions and previous positions in small increments
{
  currPos1 += (targPos1 - prevPos1) / (ird);
  currPos2 += (targPos2 - prevPos2) / (ird);
  currPos4 += (targPos4 - prevPos4) / (ird);
  currPos5 += (targPos5 - prevPos5) / (ird);
}

void setPrevPos() // set the current positions as previous positions for the next function
{
  prevPos1 = currPos1;
  prevPos2 = currPos2;
  prevPos4 = currPos4;
  prevPos5 = currPos5;
}

void updatePos() // move to new calculated positions
{
  myLSS1.move(currPos1);
  myLSS2.move(currPos2);
  myLSS4.move(currPos4);
  myLSS5.move(currPos5);
}
